c
c     MINOPEC, a routine to optimize crystal structures
c
c     by A.Gavezzotti, december 1998
c
c     *** for space groups up to orthorhombic only ***
c
c     order of parameters is a,b,c,alfa,beta,gamma,
c     3 fractional displacements of center of mass,
c     3 rigid-body rotation angles
c     ki(i)=1 optimize, ki(i)=0 do not
c     e.g for P21  111010101111
c         for P1-, Z=1  111111000111
c
c     uses symplex method
c
c     molecular rotations and translations follow the OPEC algebra
c     see Gavezzotti and Simonetta, Acta Cryst. 1977
c     cell paramater variation by deorthogonalization
c
c     input structure is OPEC format (see sample input)
c     the second line has the ki vector, 12 numbers see above
c     and an approx max molecular dimension in angstrom 
c     input file is fortran unit 11 called fort.11
c     standard output is fortran unit 9 called fort.9
c     fortran unit 12 (fort.12) has optimized structure(s) w. coordinates
c
c     the output file (fort.12) can be renamed fort.11 and
c     re-input to minopec for better convergence
c
c     both fort.11 and fort.12 can be input to program sorter
c     if further screening out of structures is desired
c
c     library potentials:   1 hydrogen   2 H-bonding hydrogen, acids
c     (kJ/mole)             3 carbon   4 nitrogen    5 oxygen
c                           6 carbonyl oxygen, acids
c                           7 carbonyl oxygen, amides
c                           8 H-bonding hydrogen, amides
c
c     for other atomic species reset species number as desired
c     and input A,B,C parameters for Aexp(-BR)-CR**-6
c     as described in the test input
c
      dimension titolo(15)
c     following dimensions are for the symplex routines
      dimension  pmat(13,12),yvec(13),psum(12),vecte(12),step(12) 
      dimension parfin(12),buffer(12),bestp(6)
      common/spl/parm(12),ndim,fract(120,3),orto(120,3)
      common/ndim/nwpe,nrm,nalf,nwatom,nalmax
      COMMON/TIPI/ WEI(30),RAVDW(30),RAVRG(30),fl(30)
      COMMON/OPEC1/ALFA(10000,3),RM(1200,3,3),QRG(120),NPE
      COMMON/OPEC2/ PESO(120),CUTOFF,ILEVEL(5),ISPEN(120),TIPO(15,15,3),
     1 NATOM,NA,NAA,NB,NC,ND,NE,NF,NG,NH,NI,NL,NM,NEE,NEM,NEF,NEL,NEP,
     2 NDD,NDE,NDF,NDG,NDH
      common/pac/ichar,ki(12),elen
      common/sing/ising,ktyp(12),ktrue(12)
      open (unit=11,file='fort.11')
      open (unit=12, file='fort.12')
      open (unit=9, file='fort.9')
      besten=0.0
c     restart new structure
 1001 continue
      ising=0
      NWPE=250
      NRM=1200
      NALF=10000
      NWATOM=120
c     nalmax is max capacity of alfa=nalf-3*nwpe-12-nwatom
      nalmax=nalf-3*nwpe-12-nwatom
C     SET ARRAY MARKERS
      NA=NWPE
      NAA=2*NWPE
      NB=NAA+1+NWPE
      NC=NB+1
      ND=NC+1
      NE=NWATOM+1
      NF=NWATOM+9
      NG=NF+1
      NH=NG
      NI=NH+NWPE
      NL=NI+NWPE
      NM=NL+NWPE
      NEE=NE+1
      NEM=NE+2
      NEF=NE+3
      NEL=NE+4
      NEP=NE+5
      NDD=ND+1
      NDE=ND+2
      NDF=ND+3
      NDG=ND+4
      NDH=ND+5
      pzz=0.01745329
c     zero everything
      do 771 i=1,13
      yvec(i)=0.0
      do 771 j=1,12
      buffer(j)=0.0
      pmat(i,j)=0.0
      psum(j)=0.0
      vecte(j)=0.0
      step(j)=0.0
      parfin(j)=0.0
      parm(j)=0.0
 771  continue
      do 772 i=1,nwatom
      peso(i)=0.0
      ispen(i)=0
      qrg(i)=0.0
      do 772 j=1,3
      fract(i,j)=0.0
 772  orto(i,j)=0.0
      do 773 i=1,nrm
      do 773 j=1,3
      do 773 k=1,3
 773  rm(i,j,k)=0.0
      do 774 i=1,nalf
      do 774 j=1,3
 774  alfa(i,j)=0.0
      READ(11,1,end=99) TITOLO                                                          
      WRITE(9,2) TITOLO                                                         
      READ(11,12) (ki(i),i=1,12),dmomax
 12   format(12i1,f10.0)
 1    FORMAT(15a4)                                                              
 2    FORMAT(/2X,'optimizing',15a4)                                                           
      READ(11,775) ilevel, cutoff
 775  format(5i1,10x,f10.0)
      READ(11,3) A,B,C,ALF,BET,GAM
      write (9,801) a,b,c,alf,bet,gam
 801  format(2x,'original cell',3f7.3,3f7.2)
  3   FORMAT(6F10.0,I5)                                                         
c     compute orthogonalization matrix
      npe=0
      call matort(a,b,c,alf,bet,gam) 
      if(ising.ne.0) go to 1001
      NATOM=0
      ichar=0
 102  READ(11,933) NUME,X,Y,Z,PEST,MES,MLD,ISPENE,QAR,RAWXX                
 933  FORMAT(I5,4F10.0,A4,I2,I3,F10.0,F5.0)                              
 932  IF(NUME-999) 100,101,101                                                  
 100  NATOM=NATOM+1                                                             
      if(ilevel(1).gt.0) write(9,1777) nume,x,y,z,pest,ispene
 1777 format(2x,i5,4f12.4,i5)
      ALFA(NUME,1)=X                                                            
      ALFA(NUME,2)=Y                                                            
      ALFA(NUME,3)=Z                                                            
      PESO(NUME)=PEST                                                           
      ISPEN(NUME)=ISPENE                                                        
      QRG(NUME)=QAR                                                             
      if(qar.ne.0.0) ichar=1
      IF(RAWXX.GT.0.01) RAVDW(ISPENE)=RAWXX                                     
      GO TO 102                                                                 
 101  continue
      K=1                                                                       
 112  READ(11,9)((RM(K,I,J),J=1,3),I=1,3)                                     
      IF(RM(K,1,1).GT.10.) GO TO 111                                            
      KO=K+NH                                                                   
      READ(11,9) (ALFA(KO,J),J=1,3)                                           
 9    FORMAT(9F5.0)                                                             
      if(ilevel(1).gt.0) WRITE(9,10)K,((RM(K,I,J),J=1,3),I=1,3),
     1 (ALFA(KO,L),L=1,3)
      K=K+1
      go to 112
 10   FORMAT(2X,I5,9F5.0,3X,3F5.2)                                              
 111  NPE=K-1                                                                   
      npind=npe
c     read potential parameters
      call reden
c     skip trailers on input file
      do 77 i=1,3
 77   read(11,*) idum
c     check original cell angles
      kcell=0
      if(alf.lt.55.0.or.alf.gt.125.0) kcell=1
      if(bet.lt.55.0.or.bet.gt.125.0) kcell=1
      if(gam.lt.55.0.or.gam.gt.125.0) kcell=1
      if(kcell.eq.0) go to 1010
      write(9,1011) a,b,c,alf,bet,gam
 1011 format(2x,3f8.3,3f8.2/2x,'oblique axes, use reduced cell')
      go to 1001
 1010 continue
c     find molecule closest to origin for better PE calculations
      do 401 k=1,natom
      do 401 i=1,3
      alfa(ne,i)=alfa(ne,i)+alfa(k,i)
 401  continue
      do 402 k=1,3
      alfa(ne,k)=alfa(ne,k)/float(natom)
 402  continue
      call prtr(nb,ne,nee)
      fmo=alfa(nee,1)**2+alfa(nee,2)**2+alfa(nee,3)**2
      fmo=sqrt(fmo)
c     write(9,420) (alfa(ne,kk),kk=1,3),
c    1 (alfa(nee,ll),ll=1,3),fmo
c420  format(2x,'original c.of.coords.',3f8.4,2x,3f9.4,/2x,
c    1 'dist.to origin',f8.2)
c     alfa(ne) has center of coords. of original molecule
      fmomin=99999.       
      do 403 i=1,npe
      call prtr(i,ne,nee)
      do 404 j=1,3
      do 404 k=1,3
      do 404 l=1,3
      alfa(nem,1)=float(j-2)
      alfa(nem,2)=float(k-2)
      alfa(nem,3)=float(l-2)
      ivec=i+nh
      call soma (nem,ivec,nem)
      call soma(nee,nem,nef) 
      call prtr(nb,nef,nef)
      fmodu=alfa(nef,1)**2+alfa(nef,2)**2+alfa(nef,3)**2
      fmodu=sqrt(fmodu)
      if(fmodu.ge.fmomin) go to 404
c     found molecule with c.o.m. closer to origin   
      fmomin=fmodu
      icomat=i  
      alfa(nel,1)=alfa(nem,1)
      alfa(nel,2)=alfa(nem,2)
      alfa(nel,3)=alfa(nem,3)
 404  continue
 403  continue
c     now perform coordinate transformation to closest point
      if(ilevel(1).ne.0)
     1write(9,410) icomat,((rm(icomat,i,k),k=1,3),i=1,3),
     1 (alfa(nel,j),j=1,3)
 410  format(2x,'setting molecule close to origin', 
     1 /2x,'transform matrix',i4,9f5.1/2x,'vector',3f7.3)
      if(ilevel(1).ne.0) write(9,421) fmomin
 421  format(2x,'new distance from origin',f10.2)
c     recalculate anyway new center of coords for use in find
      do 405 i=1,natom
      call prtr(icomat,i,i)
      call soma (i,nel,i)
      if(ilevel(1).gt.1) write(9,411) i,(alfa(i,j),j=1,3)
 411  format(2x,'atom',i5,2x,3f10.4)
 405  continue
      alfa(ne,1)=0.0
      alfa(ne,2)=0.0
      alfa(ne,3)=0.0
      do 422 k=1,natom
      do 422 i=1,3
      alfa(ne,i)=alfa(ne,i)+alfa(k,i)
 422  continue
      do 423 k=1,3
      alfa(ne,k)=alfa(ne,k)/float(natom)
 423  continue
      call prtr(nb,ne,nee)
      fmo=alfa(nee,1)**2+alfa(nee,2)**2+alfa(nee,3)**2
      fmo=sqrt(fmo)
 412  continue
      CALL MATORT(A,B,C,ALF,BET,GAM)
      if(ising.ne.0) go to 1001
c     save zero point coordinates
      do 881 i=1,natom
      do 881 j=1,3
      fract(i,j)=alfa(i,j)
      nn=i+nm
 881  orto(i,j)=alfa(nn,j)
      CALL find (dmomax)                   
      call pack (ecurr,ndi)                             
      WRITE(9,1700) ecurr,ndi                    
 1700 FORMAT(2X,'Starting energy',F12.3,2x,'over dist.',i6)
      if(ecurr.lt.300.0) go to 1002
      write(9,1003) 
      go to 1001
 1003 format(2x,'Wrong structure, optimization skipped')
 1002 continue
      CALL SETUP                                                            
c     load initial symplex
c
      parm(1)=a
      step(1)=0.005*a
      parm(2)=b
      step(2)=0.005*b
      parm(3)=c
      step(3)=0.005*c
      parm(4)=alf
      step(4)=2.0         
      parm(5)=bet
      step(5)=2.0    
      parm(6)=gam
      step(6)=2.00    
      parm(7)=0.0
      step(7)=0.1/a   
      parm(8)=0.0
      step(8)=0.1/b   
      parm(9)=0.0
      step(9)=0.1/c   
      parm(10)=0.0
      step(10)=1.5     
      parm(11)=0.0
      step(11)=1.5     
      parm(12)=0.0
      step(12)=1.5     
c     determine actual size of problem, ndim
c     vector ktrue has 1=axes,2=angles,3=transl,4=rotn       
      ndim=0     
      do 1883 i=1,3
      ktyp(i)=1
      ktyp(i+3)=2
      ktyp(i+6)=3
 1883 ktyp(i+9)=4
      do 851 i=1,12
      if(ki(i).eq.0) go to 851
      ndim=ndim+1
      ktrue(ndim)=ktyp(i)
 851  continue
 826  kk=0
      do 827 i=1,12
      if(ki(i).eq.0) go to 827
      kk=kk+1
      k=0
      do 828 j=1,12
      if(ki(j).eq.0) go to 828
      k=k+1
      pmat(kk,k)=parm(j)
      if(i.eq.j) pmat(kk,k)=pmat(kk,k)+step(j)
 828  continue
 827  continue
      mp=ndim+1
      np=ndim
      k=0
      do 830 i=1,12
      if(ki(i).eq.0) go to 830
      k=k+1
      pmat(ndim+1,k)=parm(i)
      if(k.eq.1) pmat(ndim+1,k)=parm(i)-step(i)
      if(k.eq.ndim) pmat(ndim+1,k)=pmat(ndim+1,k)-step(i)
 830  continue
      do 829 i=1,mp     
      do 840 j=1,np
 840  vecte(j)=pmat(i,j)
      call core (vecte,ecurr) 
      if(ising.ne.0) go to 1001
      yvec(i)=ecurr
 831  format(1x,12f7.2,2x,f10.2)
      if(ilevel(1).gt.0) write(9,831) (pmat(i,j),j=1,ndim),yvec(i)
 829  continue
      itmax=300
      ftol=0.0001
      iter=0
 91   do 912 n=1,ndim
      sum=0.
      do 911 m=1,ndim+1
      sum=sum+pmat(m,n)
911   continue
      psum(n)=sum
912   continue
92    ilo=1
      if (yvec(1).gt.yvec(2)) then
      ihi=1
      inhi=2
      else
      ihi=2
      inhi=1
      endif
      do 913 i=1,ndim+1
      if(yvec(i).le.yvec(ilo)) ilo=i
      if(yvec(i).gt.yvec(ihi)) then
      inhi=ihi
      ihi=i
      else if(yvec(i).gt.yvec(inhi)) then
      if(i.ne.ihi) inhi=i
      endif
913   continue
      rtol=2.*abs(yvec(ihi)-yvec(ilo))/(abs(yvec(ihi))+abs(yvec(ilo)))
      if (rtol.lt.ftol) then
      swap=yvec(1)
      yvec(1)=yvec(ilo)
      yvec(ilo)=swap
      do 914 n=1,ndim
      swap=pmat(1,n)
      pmat(1,n)=pmat(ilo,n)
      pmat(ilo,n)=swap
914   continue
      write(9,935) iter
 935  format(2x,'Exiting the symplex loop, convergence',i6)
      go to 999
      endif
      if (iter.ge.ITMAX) go to 991       
      iter=iter+2
      ytry=shift(pmat,yvec,psum,ndim,ihi,-1.0)
      if(ising.ne.0) go to 1001
      if (ytry.le.yvec(ilo)) then
      ytry=shift(pmat,yvec,psum,ndim,ihi,2.0)
      if(ising.ne.0) go to 1001
      else if (ytry.ge.yvec(inhi)) then
      ysave=yvec(ihi)
      ytry=shift(pmat,yvec,psum,ndim,ihi,0.5)
      if(ising.ne.0) go to 1001
      if (ytry.ge.ysave) then
      do 916 i=1,ndim+1
      if(i.ne.ilo)then
      do 915 j=1,ndim
      psum(j)=0.5*(pmat(i,j)+pmat(ilo,j))
c     restrict cell angle variation
      if(ktrue(j).ne.2) go to 922
      if(psum(j).lt.55.0) psum(j)=55.0
      if(psum(j).gt.125.0) psum(j)=125.0
 922  pmat(i,j)=psum(j)
915   continue
      call core (psum,ecurr)
      if(ising.ne.0) go to 1001
      yvec(i)=ecurr
      endif
916   continue
      iter=iter+ndim
      goto 91
      endif
      else
      iter=iter-1
      endif
      goto 92
 999  continue
c     end symplex loop
      k=0
      do 925 i=1,12   
      if(ki(i).eq.0) go to 926
      k=k+1
      parfin(i)=pmat(1,k)
      go to 925
 926  parfin(i)=parm(i)
 925  continue
      do 927 i=1,ndim
 927  buffer(i)=pmat(1,i)
      call core (buffer,ecurr)
      if(ising.ne.0) go to 1001
      pzz=0.0174533
      VCEL=COS(parfin(4)*PZZ)**2+COS(parfin(5)*PZZ)**2
     1 +COS(parfin(6)*PZZ)**2
      VCEL=1.-VCEL+2.*COS(parfin(4)*PZZ)*COS(parfin(5)*PZZ)
     1 *COS(parfin(6)*PZZ)
       VCEL=SQRT(VCEL)
      VCEL=VCEL*parfin(1)*parfin(2)*parfin(3)
      write(9,601) (parfin(i),i=1,6),vcel,ecurr,elen
 601  format(5x,'final cell',3f7.3,1x,3f7.2,2x,f8.1,   
     1 'etot,elen',f8.2,f8.2)
      write(9,602) (parfin(i),i=7,12)
 602  format(2x,'transl. and rotnl. shifts',2x,3f6.3,2x,3f5.1)
c     find best energy
      if(ecurr.gt.besten) go to 181
      besten=ecurr
      do 182 i=1,6
 182  bestp(i)=parfin(i)
 181  continue
c     produce final coordinate file
      if(ecurr.lt.0.0) go to 1004
      write(9,1005) 
 1005 format(2x,'Wrong structure, not written on output file')
      go to 1001
 1004 continue
      write(12,81) vcel,ecurr  
      write(12,82) (ki(i),i=1,12),dmomax
      write(12,83) 0,0,0,0,0,31,0,cutoff,cutoff
      write(12,84) (parfin(i),i=1,6)
 81   format(2x,'optimized st',5x,2x,2f10.2)
 82   format(12i1,f10.3)                           
 83   format(5i1,2i5,2f10.2)
 84   format(6f10.4)
      do 85 i=1,natom
      isp=ispen(i)
      ra=ravdw(isp)
      write(12,86) i,(alfa(i,j),j=1,3),peso(i),ispen(i),qrg(i),ra
 86   format( i5,3f10.4,f10.3,4x,i5,f10.3,f5.2)
 85   continue
      write(12,86) 999
      do 87 i=1,npind
      write(12,88) ((rm(i,j,k),k=1,3),j=1,3)
 88   format(9f5.1)
      inh=i+nh
      write(12,88) (alfa(inh,j),j=1,3)
 87   continue
c     stoppers
      write(12,88) 99.
      write(12,90) 98,1,3,99,98
 90   format(i5/2i3/i3/i4)
      go to 1001                                                                
 991  write(9,921) iter
 921  format(2x,'max iter exceeded in symplex',i6)
      go to 999
c     end all structures
 99   continue
c     print best structure
      write(9,183) besten,(bestp(i),i=1,6)
 183  format(2x,'Best energy',f10.2,2x,'cell',2x,3f10.3,3f10.1)
      stop
      END
      FUNCTION shift(p,y,psum,ndim,ihi,fac)
      REAL shift,p(13,12),psum(12),y(13),ptry(12)
      common/sing/ising,ktyp(12),ktrue(12)
      fac1=(1.-fac)/ndim
      fac2=fac1-fac
      do 11 j=1,ndim
      ptry(j)=psum(j)*fac1-p(ihi,j)*fac2
      if(ktrue(j).ne.2) go to 11
      if(ptry(j).lt.55.0) ptry(j)=55.0
      if(ptry(j).gt.125.0) ptry(j)=125.0
11    continue
      call core(ptry,ytry) 
      if(ising.ne.0) return       
      if (ytry.lt.y(ihi)) then
      y(ihi)=ytry
      do 12 j=1,ndim
      psum(j)=psum(j)-p(ihi,j)+ptry(j)
      p(ihi,j)=ptry(j)
12    continue
      endif
      shift=ytry
      return
      END
      subroutine find (dmomax)
c     find necessary surroudning molecules
c     for the packing energy calculation
c     these same molecules are used throughout the minimization
c     without changes
      common/ndim/nwpe,nrm,nalf,nwatom,nalmax
      common/pac/ichar,ki(12),elen
      COMMON/OPEC1/ALFA(10000,3),RM(1200,3,3),QRG(120),NPE
      COMMON/OPEC2/ PESO(120),CUTOFF,ILEVEL(5),ISPEN(120),TIPO(15,15,3),
     1 NATOM,NA,NAA,NB,NC,ND,NE,NF,NG,NH,NI,NL,NM,NEE,NEM,NEF,NEL,NEP,
     2 NDD,NDE,NDF,NDG,NDH
      discr=cutoff+dmomax
      npind=npe
      nsurr= nm+natom*npind
c     on entry to find, only independent symmetry opns are known
      do 511 k=1,npind
c     zero buffers
c    but alfa(ne) and alfa(nee) have c.of coords. of first molecule
      do 521 i=ne+1,ne+5
      if(i.eq.nee) go to 521
      do 522 j=1,3
      alfa(i,j)=0.0
 522  continue
 521  continue
c     compute c.of coords. for k-th molecule in cell
c     start from alfa(ne) which has that for the first mol.
      call prtr(k,ne,nem)
c     now open loops to neighbouring cells
      do 513 jj=1,7
      alfa(nef,1)=float(jj-4)
      do 513 kk=1,7
      alfa(nef,2)=float(kk-4)
      do 513 ll=1,7
      if(jj.eq.4.and.kk.eq.4.and.ll.eq.4) go to 513
      alfa(nef,3)=float(ll-4)
c     kh is original t, nef is integer transl., nel is total
c     fractional translation
      kh=k+nh
      call soma(nef,kh,nel)  
      call soma(nem,nel,nep)
      call prtr(nb,nep,nep)
c     NEE has c. of coords. of first molecule
      call dltalf(nep,nee,rx)
      if(rx.gt.discr) go to 513
c     insert new molecule in cluster
      npe=npe+1
c     insert matrix into npe-th position
      if(npe.le.nwpe) go to 5551
      write(9,552)
 552  format(2x,'max capacity of surrounding molecules exceeded')
      stop
 5551  continue
      call prma(1,k,npe)
c     insert vector
      kvect=npe+nh
      do 531 i=1,3
 531  alfa(kvect,i)=alfa(nel,i)
      if(ilevel(1).gt.0) write(9,550) npe,((rm(npe,ku,lu),lu=1,3),
     1 ku=1,3),(alfa(kvect,mu),mu=1,3),rx
 550  format(1x,'insert mol.',i4,1x,9f6.1,1x,3f7.1,f10.3)
 513  continue
 511  continue
c     now store orthogonal coordinates of new molecule
      next=npind+1
      do 555 i=next,npe   
      do 515 j=1,natom
      call prtr(i,j,nep)
      ih=i+nh
      call soma(nep,ih,nep)
      nsurr=nsurr+1
      if(nsurr.le.nalmax) go to 516
      write(9,551)
 551  format(2x,'max capacity of alfa vector exceeded')
      stop
 516  continue
      call prtr(nb,nep,nsurr)
 515  continue
 555  continue
      return
      end
      SUBROUTINE CORE (vect,ecurr)
      DIMENSION vect(12),parz(12)                                   
      COMMON/OPEC1/ALFA(10000,3),RM(1200,3,3),QRG(120),NPE
      COMMON/OPEC2/ PESO(120),CUTOFF,ILEVEL(5),ISPEN(120),TIPO(15,15,3),
     1 NATOM,NA,NAA,NB,NC,ND,NE,NF,NG,NH,NI,NL,NM,NEE,NEM,NEF,NEL,NEP,
     2 NDD,NDE,NDF,NDG,NDH
      common/pac/ichar,ki(12),elen
      common /spl/parm(12),ndim,fract(120,3),orto(120,3)
      common/sing/ising,ktyp(12),ktrue(12)
      pzz=0.01745329
      do 887 i=1,12
 887  parz(i)=0.0
c     restore zero point coordinates
      do 888 i=1,natom
      do 888 j=1,3
      nn=i+nm
      alfa(i,j)=fract(i,j)
 888  alfa(nn,j)=orto(i,j)
c     match actual parameter vector to standard parameter vector
      k=0
      do 801 i=1,12   
      if(ki(i).ne.0) go to 802
      parz(i)=parm(i)
      go to 801
 802  k=k+1
      parz(i)=vect(k)
 801  continue
c     prepare for cell variations
      a=parz(1)
      b=parz(2)
      c=parz(3)
      alf=parz(4)
      bet=parz(5)
      gam=parz(6)
      npold=npe
      npe=0
      call matort(a,b,c,alf,bet,gam)       
      if(ising.ne.0) return       
c     derthogonalize old coordinates with new cell
      do 711 i=1,natom
      next=nm+i
      call prtr(nc,next,i) 
 711  continue
c     recalculate orthogonal coords of first molecule
      npe=1
      call matort(a,b,c,alf,bet,gam)
      if(ising.ne.0) return
c     now restore npe to its true value
      npe=npold
      do 712 i=1,natom
      j=i+nm
 712  continue
      call setup
c     prepare for rotation and translation of center of mass
      ALFA(NE,1)=parz(7)              
      ALFA(NE,2)=parz(8)              
      ALFA(NE,3)=parz(9)              
      C1=COS(parz(10)*pzz)
      S1=SIN(parz(10)*pzz)
      C2=COS(parz(11)*pzz)
      S2=SIN(parz(11)*pzz)
      C3=COS(parz(12)*pzz)
      S3=SIN(parz(12)*pzz)
      CALL MATFIL(ND,C1,S1,C2,S2,C3,S3)
      ip=1
      ip6=nl+1 
      ipk=na+1
      ipkk=naa+1
c     rm(nd) contains rotation matrix
c     alfa(ne) contains translation vector
      CALL PRMA (IPKK,ND,NDD)
      CALL PRMA (NDD,IPK,NDE)                                                   
      CALL PRTR (NDE,IP6,NEF)                                                   
      CALL PRTR (NB,NE,NEE)                                                     
      CALL SOMA (NEF,NEE,NEF)                                                   
      CALL DIF (NEF,IP6,NEF)                                                    
c     produce rotated orthogonalized coordinates
c     CALL ROTAZ                                                          
      DO 25 IBG=1,NATOM
      IK=IBG+nm
      IPPO=NI+1 
      CALL PRMA (NB,IP,NDF)
      CALL PRTR (NDF,IBG,NEE)   
      CALL SOMA (IPPO,NEE,NEE)
      CALL PRTR (NDE,NEE,IK)
      CALL SOMA (NEF,IK,IK)
 25   CONTINUE
c     now deorthogonalize 
      do 100 i=1,natom
      ind=nm+i
      call prtr(nc,ind,i)
 100  continue
c     now generate new packing coordinates
      JL=NM
      DO 2550 i=1,npe                                                           
      NIX=nh+i                                                                
      DO 2551 j=1,NATOM                                                       
      JL=JL+1                                                                   
      CALL PRMA(NB,I,NDF)                                                     
      CALL PRTR(NDF,j,JL)                                                     
      call prtr(nb,nix,nee)
      CALL SOMA (nee,jl,jl)                                                     
 2551 CONTINUE                                                                  
 2550 CONTINUE                                                                  
      CALL PACK (ecurr,ndi)                                                  
      if(ilevel(1).gt.0) write(9,17) parz,ecurr
 17   format(3f8.3,3f7.1,3f7.3,3f6.2,2x,f10.3)
      RETURN                                                                    
      END                                                                       
      SUBROUTINE PACK (ecurr,ndi)                                   
      COMMON/OPEC1/ALFA(10000,3),RM(1200,3,3),QRG(120),NPE
      COMMON/OPEC2/ PESO(120),CUTOFF,ILEVEL(5),ISPEN(120),TIPO(15,15,3),
     1 NATOM,NA,NAA,NB,NC,ND,NE,NF,NG,NH,NI,NL,NM,NEE,NEM,NEF,NEL,NEP,
     2 NDD,NDE,NDF,NDG,NDH
      common/pac/ichar,ki(12),elen
      cut2=cutoff*cutoff   
      ecurr=0.0
      ndi=0
      npem=npe-1
      do 2 i=1,natom
      nsu=i+nm
      ispi=ispen(i)
      do 3 j=1,npem
      next=nm+j*natom
      do 4 k=i,natom
      indir=next+k
      r1=alfa(nsu,1)-alfa(indir,1)
      r2=alfa(nsu,2)-alfa(indir,2)
      r3=alfa(nsu,3)-alfa(indir,3)
      r=r1*r1+r2*r2+r3*r3
      if(r.gt.cut2) go to 4
      if(r.gt.1.0) go to 5
      write(9,6) i,k,npem,r
 6    format(2x,'atom',i4,1x,'atom',i4,'surr.mol.no.',
     1 i4,f10.2,2x,'short r')
      ecurr=1000.
      return    
 5    ispk=ispen(k)
      ndi=ndi+1
      r=sqrt(r)
      ecu=tipo(ispi,ispk,1)*exp(-tipo(ispi,ispk,2)*r)
      ecu=ecu-tipo(ispi,ispk,3)*r**(-6)
      if(i.eq.k) ecu=0.5*ecu
      ecurr=ecurr+ecu
 4    continue
 3    continue
 2    continue
      if(ichar.eq.0) return
c     point-charge energy section 
      elen=0.0
      npem=npe-1
      DO 121 I=1,NATOM
      JL=NM+I
      Q1=QRG(i)
      IF(Q1.EQ.0.0) GO TO 121
      DO 122 J=1,npem
      next=nm+j*natom
      DO 122 K=i,NATOM
      indir=next+k
      Q2=QRG(k)
      CALL DLTALF(JL,indir,rq)
      e1=q1*q2/rq
      if(i.eq.k) e1=0.5*e1
      elen=elen+e1
 122  CONTINUE
 121  CONTINUE
      elen=elen*332.2*4.184
      if(ilevel(1).gt.0) write(9,131) elen
 131  format(2x,'point-charge energy kJ/mole',2x,f10.3)
      ecurr=ecurr+elen
      return
      END                                                                       
      SUBROUTINE MATFIL(I,C1,S1,C2,S2,C3,S3)
      COMMON/OPEC1/ALFA(10000,3),RM(1200,3,3),QRG(120),NPE
      RM(I,1,1)=C2*C3
      RM(I,1,2)=-C1*S3+S1*S2*C3
      RM(I,1,3)=S1*S3+C1*S2*C3
      RM(I,2,1)=C2*S3
      RM(I,2,2)=C1*C3+S1*S2*S3
      RM(I,2,3)=-S1*C3+C1*S2*S3
      RM(I,3,1)=-S2
      RM(I,3,2)=S1*C2
      RM(I,3,3)=C1*C2
 5    RETURN
      END
      SUBROUTINE DLTALF(I,J,RRXX)
      COMMON/OPEC1/ALFA(10000,3),RM(1200,3,3),QRG(120),NPE
      RRXX=(ALFA(I,1)-ALFA(J,1))**2+
     1     (ALFA(I,2)-ALFA(J,2))**2+(ALFA(I,3)-ALFA(J,3))**2
      RRXX=SQRT(RRXX)
      RETURN
      END
      SUBROUTINE DIF (J,K,I)
      COMMON/OPEC1/ALFA(10000,3),RM(1200,3,3),QRG(120),NPE
      ALFA(I,1)=ALFA(J,1)-ALFA(K,1)
      ALFA(I,2)=ALFA(J,2)-ALFA(K,2)
      ALFA(I,3)=ALFA(J,3)-ALFA(K,3)
      RETURN
      END
      SUBROUTINE REDEN 
      DIMENSION ITY(15)
      COMMON/OPEC2/ PESO(120),CUTOFF,ILEVEL(5),ISPEN(120),TIPO(15,15,3),
     1 NATOM,NA,NAA,NB,NC,ND,NE,NF,NG,NH,NI,NL,NM,NEE,NEM,NEF,NEL,NEP,
     2 NDD,NDE,NDF,NDG,NDH
      COMMON/TIPI/ WEI(30),RAVDW(30),RAVRG(30),FL(30)             
C     LOAD POTENTIAL PARAMETER LIBRARY
      CALL PPOTE
 11   READ(11,1) I,J,A,B,C
      IF(I.EQ.98) GO TO 50
      if(i.gt.8.or.j.gt.8) call err
c     WRITE(6,4)
 4    FORMAT  (2X,25HINPUT INTERACTION CURVES,,2X,9HSPECIES  ,
     *5X,5HA,B,C)
 1    FORMAT(2I5,3F10.0)
      TIPO(I,J,1)=A
      TIPO(I,J,2)=B
      TIPO(I,J,3)=C
      TIPO(J,I,1)=A
      TIPO(J,I,2)=B
      TIPO(J,I,3)=C
c     WRITE(6,5) I,J,(TIPO(I,J,K),K=1,3)
 5    FORMAT(2X,2I5,2X,F10.2,F8.3,F8.3)
      GO TO 11
 50   IND=0
C     PROVIDE LIBRARY PARAMETERS
      DO 30  I=1,15
 30   ITY(I)=0
      DO 20 I=1,NATOM
      ISO=ISPEN(I)
      DO 21 J=1,15
      IF(ISO.EQ.ITY(J)) GO TO 20
 21   CONTINUE
      IND=IND+1
      ITY(IND)=ISO
 20   CONTINUE
c     WRITE(6,22)
 22   FORMAT  (/' POTENTIAL PARAMETERS, Aexp(-Br-Cr**-6')
      DO 31 I=1,IND
      IS1=ITY(I)
      DO 32 J=I,IND
      IS2=ITY(J)
      ISTIP=TIPO(IS1,IS2,1)
      IF(ISTIP.GT.0) GO TO 33
c     WRITE(6,34) IS1,IS2
 34   FORMAT (2X,'Supply missing parameters for',2I5)
      TIPO(IS1,IS2,1)=(TIPO(IS1,IS1,1)*TIPO(IS2,IS2,1))**0.5
      TIPO(IS1,IS2,3)=(TIPO(IS1,IS1,3)*TIPO(IS2,IS2,3))**0.5
      TIPO(IS1,IS2,2)=    (TIPO(IS1,IS1,2)+TIPO(IS2,IS2,2))*0.5
      DO 37 K=1,3
 37   TIPO(IS2,IS1,K)=TIPO(IS1,IS2,K)
 33   continue
c     WRITE(6,35) IS1,IS2,(TIPO(IS1,IS2,L),L=1,3)
 35   FORMAT(5X,2I5,2X,F12.2,F8.3,F8.2)
 32   CONTINUE
 31   CONTINUE
      RETURN
      END
C     THIS SUBROUTINE ASSIGNS POTENTIAL ENERGY PARAMETERS
C     VERSION OF JANUARY 1998
C     1 H   2 H ACIDS   3 C   4 N   5 O
C     6 =O ACIDS     7 =O AMIDES    8 H AMIDES
      SUBROUTINE PPOTE
      COMMON/OPEC2/ PESO(120),CUTOFF,ILEVEL(5),ISPEN(120),TIPO(15,15,3),
     1 NATOM,NA,NAA,NB,NC,ND,NE,NF,NG,NH,NI,NL,NM,NEE,NEM,NEF,NEL,NEP,
     2 NDD,NDE,NDF,NDG,NDH
      DO 1 I=1,15
      DO 1 J=1,15
      DO 1 K=1,3
 1    TIPO(I,J,K)=0.0
      TIPO(1,1,1)=    24158.42
      TIPO(1,1,2)=        4.01
      TIPO(1,1,3)=      109.20
      TIPO(1,2,1)=    24158.42
      TIPO(1,2,2)=        4.01
      TIPO(1,2,3)=      109.20
      TIPO(1,3,1)=   120792.10
      TIPO(1,3,2)=        4.10
      TIPO(1,3,3)=      472.79
      TIPO(1,4,1)=   228279.00
      TIPO(1,4,2)=        4.52
      TIPO(1,4,3)=      502.08
      TIPO(1,5,1)=   295432.30
      TIPO(1,5,2)=        4.82
      TIPO(1,5,3)=      439.32
      TIPO(1,6,1)=   295432.30
      TIPO(1,6,2)=        4.82
      TIPO(1,6,3)=      439.32
      TIPO(1,7,1)=   295432.30
      TIPO(1,7,2)=        4.82
      TIPO(1,7,3)=      439.32
      TIPO(1,8,1)=    24158.42
      TIPO(1,8,2)=        4.01
      TIPO(1,8,3)=      109.20
      TIPO(2,2,1)=    24158.42
      TIPO(2,2,2)=        4.01
      TIPO(2,2,3)=      109.20
      TIPO(2,3,1)=   120792.10
      TIPO(2,3,2)=        4.10
      TIPO(2,3,3)=      472.79
      TIPO(2,4,1)=   228279.00
      TIPO(2,4,2)=        4.52
      TIPO(2,4,3)=      502.08
      TIPO(2,5,1)=   295432.30
      TIPO(2,5,2)=        4.82
      TIPO(2,5,3)=      439.32
      TIPO(2,6,1)= 26416400.00
      TIPO(2,6,2)=        8.75
      TIPO(2,6,3)=      857.72
      TIPO(3,3,1)=   226145.20
      TIPO(3,3,2)=        3.47
      TIPO(3,3,3)=     2418.35
      TIPO(3,4,1)=   491494.50
      TIPO(3,4,2)=        3.86
      TIPO(3,4,3)=     2790.73
      TIPO(3,5,1)=   393086.80
      TIPO(3,5,2)=        3.74
      TIPO(3,5,3)=     2681.94
      TIPO(3,6,1)=   393086.80
      TIPO(3,6,2)=        3.74
      TIPO(3,6,3)=     2681.94
      TIPO(3,7,1)=   393086.80
      TIPO(3,7,2)=        3.74
      TIPO(3,7,3)=     2681.94
      TIPO(3,8,1)=   120792.10
      TIPO(3,8,2)=        4.10
      TIPO(3,8,3)=      472.79
      TIPO(4,4,1)=   365263.20
      TIPO(4,4,2)=        3.65
      TIPO(4,4,3)=     2891.14
      TIPO(4,5,1)=   268571.00
      TIPO(4,5,2)=        3.86
      TIPO(4,5,3)=     1522.98
      TIPO(4,6,1)=   268571.00
      TIPO(4,6,2)=        3.86
      TIPO(4,6,3)=     1522.98
      TIPO(4,7,1)=   268571.00
      TIPO(4,7,2)=        3.86
      TIPO(4,7,3)=     1522.98
      TIPO(4,8,1)=   228279.00
      TIPO(4,8,2)=        4.52
      TIPO(4,8,3)=      502.08
      TIPO(5,5,1)=   195309.10
      TIPO(5,5,2)=        3.74
      TIPO(5,5,3)=     1334.70
      TIPO(5,6,1)=   195309.10
      TIPO(5,6,2)=        3.74
      TIPO(5,6,3)=     1334.70
      TIPO(5,7,1)=   195309.10
      TIPO(5,7,2)=        3.74
      TIPO(5,7,3)=     1334.70
      TIPO(5,8,1)=   295432.30
      TIPO(5,8,2)=        4.82
      TIPO(5,8,3)=      439.32
      TIPO(6,6,1)=   195309.10
      TIPO(6,6,2)=        3.74
      TIPO(6,6,3)=     1334.70
      TIPO(6,7,1)=   195309.10
      TIPO(6,7,2)=        3.74
      TIPO(6,7,3)=     1334.70
      TIPO(7,7,1)=   195309.10
      TIPO(7,7,2)=        3.74
      TIPO(7,7,3)=     1334.70
      TIPO(7,8,1)= 15095080.00
      TIPO(7,8,2)=        7.78
      TIPO(7,8,3)=      995.79
      TIPO(8,8,1)=    24158.42
      TIPO(8,8,2)=        4.01
      TIPO(8,8,3)=      109.20
C     SYMMETRIZE POTENTIAL PARAMETER MATRIX
      DO 2 I=1,15
      DO 2 J=I,15
      DO 2 K=1,3
 2    TIPO(J,I,K)=TIPO(I,J,K)
      RETURN
      END
      BLOCK DATA BLKD11
      COMMON/TIPI/ WEI(30),RAVDW(30),RAVRG(30),FL(30)
      COMMON/OPEC2/ PESO(120),CUTOFF,ILEVEL(5),ISPEN(120),TIPO(15,15,3),
     1 NATOM,NA,NAA,NB,NC,ND,NE,NF,NG,NH,NI,NL,NM,NEE,NEM,NEF,NEL,NEP,
     2 NDD,NDE,NDF,NDG,NDH
      DATA WEI/1.0,1.0,12.,14.,16.,16.,16.,1.0,35.5,21*0.0/        
      DATA RAVRG/0.3,0.3,0.67,0.63,0.66,0.66,0.66,0.30,0.8,21*0.0/
      DATA RAVDW/1.17,1.17,1.75,1.55,1.4,1.4,1.4,1.17,1.8,21*0.0/     
      DATA FL/' H',' H',' C',' N',' O',' O',' O',' H','CL',21*'  '/   
      END
      SUBROUTINE BARIC(NATOM,PESO,NF)
C     COMPUTE CENTER OF MASS
      DIMENSION PESO(120)
      COMMON/OPEC1/ALFA(10000,3),RM(1200,3,3),QRG(120),NPE
      DO 11 I=1,3
 11   ALFA(NF,I)= 0.0
      SUMS=0.
      DO 500 I=1,NATOM
      PESI=PESO(I)
      SUMS=SUMS+PESI
      ALFA(NF,1)=ALFA(NF,1)+ALFA(I,1)*PESI
      ALFA(NF,2)=ALFA(NF,2)+ALFA(I,2)*PESI
      ALFA(NF,3)=ALFA(NF,3)+ALFA(I,3)*PESI
  500 CONTINUE
      ALFA(NF,1)=ALFA(NF,1)/SUMS
      ALFA(NF,2)=ALFA(NF,2)/SUMS
      ALFA(NF,3)=ALFA(NF,3)/SUMS
      RETURN
      END
      SUBROUTINE INV(I,J)
      COMMON/OPEC1/ALFA(10000,3),RM(1200,3,3),QRG(120),NPE
      R11=RM(I,1,1)
      R12=RM(I,1,2)
      R13=RM(I,1,3)
      R21=RM(I,2,1)
      R22=RM(I,2,2)
      R23=RM(I,2,3)
      R31=RM(I,3,1)
      R32=RM(I,3,2)
      R33=RM(I,3,3)
      DET=R11*R22*R33+R12*R23*R31+R21*R32*R13-R13*R22*R31
      DET=DET-R11*R23*R32-R12*R21*R33
      RM(J,1,1)=(R22*R33-R23*R32)/DET
      RM(J,2,1)=-(R21*R33-R23*R31)/DET
      RM(J,3,1)=(R21*R32-R22*R31)/DET
      RM(J,1,2)=-(R12*R33-R13*R32)/DET
      RM(J,2,2)=(R11*R33-R13*R31)/DET
      RM(J,3,2)=-(R11*R32-R12*R31)/DET
      RM(J,1,3)=(R12*R23-R13*R22)/DET
      RM(J,2,3)=-(R11*R23-R13*R21)/DET
      RM(J,3,3)=(R11*R22-R12*R21)/DET
      RETURN
      END
      SUBROUTINE JACOB (MIX,NIX)
C     DIAGONALIZE MATRIX RM(MIX),PUT EIGENVECTORS IN RM(NIX)
      DIMENSION F(3,3),V(3,3)
      COMMON/OPEC1/ALFA(10000,3),RM(1200,3,3),QRG(120),NPE
      COMMON/OPEC2/ PESO(120),CUTOFF,ILEVEL(5),ISPEN(120),TIPO(15,15,3),
     1 idum(23)     
      DOUBLE PRECISION SINT,COST,FUNNO,FUNT
      N=3
      RHO=0.000001
      DO 50 I=1,3
      DO 50 J=1,3
      F(I,J)=RM(MIX,I,J)
 50   CONTINUE
      TE=0.0
      FUNNO=1.0
      DO 20 I=1,3
      DO 21 J=1,3
      V(I,J)=0.0
 21   CONTINUE
      V(I,I)=1.0
 20   CONTINUE
      DO 1 I=2,N
      K=I-1
      DO 1 J=1,K
      TE=TE+2.*F(I,J)*F(I,J)
 1    CONTINUE
      TE=SQRT(TE)
      MA=0
      A=N
    2 TE=TE/A
      IF(TE.LT.RHO)TE=RHO
    3 DO 14 II=2,N
      IJ=II-1
      DO 14 JJ=1,IJ
      IF (ABS(F(II,JJ))-TE)14,4,4
    4 MA=1
      V1=F(JJ,JJ)
      V2=F(II,JJ)
      V3=F(II,II)
      U=.5*(F(JJ,JJ)-F(II,II))
      IF(ABS(U)-RHO)5,6,6
    5 OMEGA=-1.0
      GO TO 7
   6  FUX=SQRT(F(II,JJ)*F(II,JJ)+U*U)
      OMEGA=-F(II,JJ)/FUX
      Z=1.
      IF (U.LT.0.)Z=-Z
      OMEGA=OMEGA*Z
 7    FUX=1.0-OMEGA*OMEGA
      FEX=2.0*(1.0+SQRT(FUX))
      SINT=OMEGA/SQRT(FEX)
      FUNT=FUNNO-SINT*SINT
      COST=SQRT(FUNT)
      DO 13 I=1,N
      IF(I-II)9,8,8
    8 TEM=F(I,JJ)*COST-F(I,II)*SINT
      F(I,II)=F(I,JJ)*SINT+F(I,II)*COST
      F(I,JJ)=TEM
      GO TO 12
    9 IF(I-JJ)10,11,11
   10 TEM=F(JJ,I)*COST-F(II,I)*SINT
      F(II,I)=F(JJ,I)*SINT+F(II,I)*COST
      F(JJ,I)=TEM
      GO TO 12
   11 TEM=F(I,JJ)*COST-F(II,I)*SINT
      F(II,I)=F(I,JJ)*SINT+F(II,I)*COST
      F(I,JJ)=TEM
   12 TEM=V(I,JJ)*COST-V(I,II)*SINT
      V(I,II)=V(I,JJ)*SINT+V(I,II)*COST
      V(I,JJ)=TEM
 13   CONTINUE
      F(JJ,JJ)=V1*COST*COST+V3*SINT*SINT-2.*V2*SINT*COST
      F(II,II)=V1*SINT*SINT+V3*COST*COST+2.*V2*SINT*COST
      F(II,JJ)=2.*U*SINT*COST+V2*(COST*COST-SINT*SINT)
   14 CONTINUE
      IF(MA-1)16,15,15
   15 MA=0
      GO TO 3
   16 IF(TE-RHO)17,17,2
C     EIGENVALUE ORDERING
 17   CONTINUE
 310  IF(F(1,1)-F(2,2)) 100,101,101
 101  IF(F(2,2)-F(3,3)) 102,102,103
 103  IND=6
      GO TO 200
 102  IF(F(1,1)-F(3,3)) 104,104,105
 104  IND=2
      GO TO 200
 105  IND=5
      GO TO 200
 100  IF(F(2,2)-F(3,3)) 107,107,108
 107  IND=1
      GO TO 200
 108  IF(F(1,1)-F(3,3)) 109,109,110
 109  IND=4
      GO TO 200
 110  IND=3
 200  GO TO (116,115,116,115,115,201),IND
 115  F33=F(2,2)
      F(2,2)=F(3,3)
      F(3,3)=F33
      DO 120 I=1,3
      VC=V(I,3)
      V(I,3)=V(I,2)
 120  V(I,2)=VC
 116  IF(IND.EQ.5) GO TO 201
      GO TO (118,119,119,118) ,IND
 118  F33=F(1,1)
      F(1,1)=F(3,3)
      F(3,3)=F33
      DO 220 I=1,3
      VC=V(I,3)
      V(I,3)=V(I,1)
 220  V(I,1)=VC
      GO TO 201
 119  F33=F(2,2)
      F(2,2)=F(1,1)
      F(1,1)=F33
      DO 221 I=1,3
      VC=V(I,1)
      V(I,1)=V(I,2)
 221  V(I,2)=VC
 201  DO 51 I=1,3
      DO 51 J=1,3
      RM(MIX,I,J)=0.0
 51   RM(NIX,I,J)=V(J,I)
      RETURN
       END
      SUBROUTINE MATORT (A,B,C,ALF,BET,GAM)       
C     COMPUTE ORTHOGONALIZATION MATRIX)
      COMMON/OPEC1/ALFA(10000,3),RM(1200,3,3),QRG(120),NPE
      COMMON/OPEC2/ PESO(120),CUTOFF,ILEVEL(5),ISPEN(120),TIPO(15,15,3),
     1 NATOM,NA,NAA,NB,NC,ND,NE,NF,NG,NH,NI,NL,NM,NEE,NEM,NEF,NEL,NEP,
     2 NDD,NDE,NDF,NDG,NDH
      common/ndim/nwpe,nrm,nalf,nwatom,nalmax
      common/sing/ising,ktyp(12),ktrue(12)
      DO 20 I=1,3
 20   ALFA (NF,I)=0.0
      PZ=0.01745329
      CALF=COS(ALF*PZ)
      SALF=SIN(ALF*PZ)
      CBET=COS(BET*PZ)
      SBET=SIN(BET*PZ)
      CGAM=COS(GAM*PZ)
      SGAM=SIN(GAM*PZ)
      RM(NB,1,1)=A*SGAM
      RM(NB,1,2)=0.0
      RM(NB,1,3)=C/SGAM
      RM(NB,2,1)=A*CGAM
      RM(NB,2,2)=B
      RM(NB,2,3)=CALF
      RM(NB,3,1)=0.0
      RM(NB,3,2)=0.0
      RM(NB,3,3)=1.-CGAM*CGAM-CBET*CBET-RM(NB,2,3)**2
     1 +2.*CGAM*CBET*RM(NB,2,3)
c     check for singularities in cell and orthogonalization matrix
      if(rm(nb,3,3).gt.0.05) go to 444
      write(9,446) a,b,c,alf,bet,gam
 446  format(15x,'cell',3f8.3, 3f8.2)
      write(9,445) rm(nb,3,3)
 445  format(2x,'orthogonalization matrix discriminant',e12.4,
     1 /2x,'cannot continue-restart minopec with better reduced cell')
      ising=1
      return   
 444  continue
      RM(NB,3,3)=RM(NB,1,3)*SQRT(RM (NB,3,3))
      RM(NB,1,3)=RM(NB,1,3)*(CBET-CGAM*RM(NB,2,3))
      RM(NB,2,3)=RM(NB,2,3)*C
      CALL INV (NB,NC)
      if(npe.eq.0) return
C     PREPARE SYMMETRY-RELATED MOLECULES
      DO 503 I=1,NPE                                                            
      IK=I+NH                                                                   
      IKK=I+NI                                                                  
 503  CALL PRTR (NB,IK,IKK)                                                     
      JL=NM                                                                     
      DO 510 I=1,NPE                                                            
      IK=I+NI                                                                   
      DO 511 J=1,NATOM                                                          
      JL=JL+1                                                                   
 21   CALL PRMA (NB,I,NDF)                                                      
      CALL PRTR (NDF,J,JL)                                                      
      CALL SOMA(JL,IK,JL)                                                       
 511  CONTINUE                                                                  
 510  CONTINUE                                                                  
      CALL BARIC(NATOM,PESO,NF)
      CALL PRTR (NB,NF,NG)
      RETURN
      END
      SUBROUTINE PRMA(J,K,I)
      COMMON/OPEC1/ALFA(10000,3),RM(1200,3,3),QRG(120),NPE
      A11=RM(J,1,1)*RM(K,1,1)+RM(J,1,2)*RM(K,2,1)+RM(J,1,3)*RM(K,3,1)
      A12=RM(J,1,1)*RM(K,1,2)+RM(J,1,2)*RM(K,2,2)+RM(J,1,3)*RM(K,3,2)
      A13=RM(J,1,1)*RM(K,1,3)+RM(J,1,2)*RM(K,2,3)+RM(J,1,3)*RM(K,3,3)
      A21=RM(J,2,1)*RM(K,1,1)+RM(J,2,2)*RM(K,2,1)+RM(J,2,3)*RM(K,3,1)
      A22=RM(J,2,1)*RM(K,1,2)+RM(J,2,2)*RM(K,2,2)+RM(J,2,3)*RM(K,3,2)
      A23=RM(J,2,1)*RM(K,1,3)+RM(J,2,2)*RM(K,2,3)+RM(J,2,3)*RM(K,3,3)
      A31=RM(J,3,1)*RM(K,1,1)+RM(J,3,2)*RM(K,2,1)+RM(J,3,3)*RM(K,3,1)
      A32=RM(J,3,1)*RM(K,1,2)+RM(J,3,2)*RM(K,2,2)+RM(J,3,3)*RM(K,3,2)
      A33=RM(J,3,1)*RM(K,1,3)+RM(J,3,2)*RM(K,2,3)+RM(J,3,3)*RM(K,3,3)
      RM(I,1,1)=A11
      RM(I,1,2)=A12
      RM(I,1,3)=A13
      RM(I,2,1)=A21
      RM(I,2,2)=A22
      RM(I,2,3)=A23
      RM(I,3,1)=A31
      RM(I,3,2)=A32
      RM(I,3,3)=A33
      RETURN
      END
      SUBROUTINE PRTR (I,J,K)
      DIMENSION UG(3)
      COMMON/OPEC1/ALFA(10000,3),RM(1200,3,3),QRG(120),NPE
      UG(1)=RM(I,1,1)*ALFA(J,1)+RM(I,1,2)*ALFA(J,2)+RM(I,1,3)*ALFA(J,3)
      UG(2)=RM(I,2,1)*ALFA(J,1)+RM(I,2,2)*ALFA(J,2)+RM(I,2,3)*ALFA(J,3)
      UG(3)=RM(I,3,1)*ALFA(J,1)+RM(I,3,2)*ALFA(J,2)+RM(I,3,3)*ALFA(J,3)
      ALFA(K,1)=UG(1)
      ALFA(K,2)=UG(2)
      ALFA(K,3)=UG(3)
      RETURN
      END
      SUBROUTINE SETUP  
      COMMON/OPEC1/ALFA(10000,3),RM(1200,3,3),QRG(120),NPE
      COMMON/OPEC2/ PESO(120),CUTOFF,ILEVEL(5),ISPEN(120),TIPO(15,15,3),
     1 NATOM,NA,NAA,NB,NC,ND,NE,NF,NG,NH,NI,NL,NM,NEE,NEM,NEF,NEL,NEP,
     2 NDD,NDE,NDF,NDG,NDH
      PZ=0.01745329
      i=1
      C1=0.0
      C2=0.0
      C3=0.0
      SOMX=0.0
      SOMY=0.0
      SOMZ=0.0
      IKU=I+NL
      ICE=I+NI
      CALL BARIC(NATOM,PESO,NF)
      CALL PRMA (NB,I,NDF)
      CALL PRTR (NDF,NF,NEE)
      CALL SOMA (NEE,ICE,IKU)
      ALFA(IKU,1)=-ALFA(IKU,1)
      ALFA(IKU,2)=-ALFA(IKU,2)
      ALFA(IKU,3)=-ALFA(IKU,3)
      DO 880 KK=1,NATOM
      TUNE=PESO(KK)
      IF(I-1) 880,882,882
 882  KKI=(I-1)*NATOM+NM+KK
 803  CALL SOMA (KKI,IKU,NEL)
      C1=C1+ALFA(NEL,1)*TUNE*ALFA(NEL,1)
      C2=C2+ALFA(NEL,2)*TUNE*ALFA(NEL,2)
      C3=C3+ALFA(NEL,3)*TUNE*ALFA(NEL,3)
      SOMX=SOMX-ALFA(NEL,1)*ALFA(NEL,2)*TUNE
      SOMY=SOMY-ALFA(NEL,1)*ALFA(NEL,3)*TUNE
      SOMZ=SOMZ-ALFA(NEL,2)*ALFA(NEL,3)*TUNE
 880  CONTINUE
      RM(NDH,1,1)=C2+C3
      RM(NDH,2,2)=C1+C3
      RM(NDH,3,3)=C1+C2
      RM(NDH,1,2)=SOMX
      RM(NDH,1,3)=SOMY
      RM(NDH,2,3)=SOMZ
      RM(NDH,2,1)=RM(NDH,1,2)
      RM(NDH,3,1)=RM(NDH,1,3)
      RM(NDH,3,2)=RM(NDH,2,3)
      IKKO=I+NA
      CALL JACOB (NDH,IKKO)
      RM(IKKO,3,1)=RM(IKKO,1,2)*RM(IKKO,2,3)-RM(IKKO,1,3)*RM(IKKO,2,2)
      RM(IKKO,3,2)=RM(IKKO,1,3)*RM(IKKO,2,1)-RM(IKKO,1,1)*RM(IKKO,2,3)
      RM(IKKO,3,3)=RM(IKKO,1,1)*RM(IKKO,2,2)-RM(IKKO,1,2)*RM(IKKO,2,1)
      inve=i+naa
      call inv(ikko,inve)
      RETURN
      END
      SUBROUTINE SOMA (I,J,K)
      COMMON/OPEC1/ALFA(10000,3),RM(1200,3,3),QRG(120),NPE
      ALFA(K,1)=ALFA(I,1)+ALFA(J,1)
      ALFA(K,2)=ALFA(I,2)+ALFA(J,2)
      ALFA(K,3)=ALFA(I,3)+ALFA(J,3)
      RETURN
      END
      subroutine err
      write(9,1)
   1  format(2x,'something wrong, program cannot continue')
      stop
      end

